<?php

// just making sure the constant is defined
if (!defined('WP_CONTENT_DIR'))
    define('WP_CONTENT_DIR', ABSPATH . 'wp-content');
 
require_once('ASDEnvironment.class.php');

if (!class_exists('WPASDPlugin')) {
    abstract class WPASDPlugin {
        protected $ASDEnvironment; // what ASDEnvironment are we in
        protected $options_name; // the name of the options associated with this plugin
        
        protected $options;
        
        function WPASDPlugin($options_name) {
            $args = func_get_args();
            call_user_func_array(array(&$this, "__construct"), $args);
        }
        
        function __construct($options_name) {
            $this->ASDEnvironment = WPASDPlugin::determine_ASDEnvironment();
            $this->options_name = $options_name;
            
            $this->options = WPASDPlugin::retrieve_options($this->options_name);
	    
	    $this->register_initialization();
        }
	
	protected function register_initialization() {
	
	    add_action('plugins_loaded', array(&$this, 'initialize'));
	
	}
	
	function initialize() {
	
	    $this->pre_init();
	
	    $this->register_actions();
	    $this->register_filters();
	    
	    $this->post_init();
	    
	}
	
	abstract protected function pre_init();
	abstract protected function post_init();
        
        // sub-classes determine what actions and filters to hook
        abstract protected function register_actions();
        abstract protected function register_filters();
        
        // ASDEnvironment checking
        static function determine_ASDEnvironment() {
            global $wpmu_version;
            
            if (function_exists('is_multisite'))
                if (is_multisite())
                    return ASDEnvironment::WordPressMS;
            
            if (!empty($wpmu_version))
                return ASDEnvironment::WordPressMU;
                
            return ASDEnvironment::WordPress;
        }
        
        // path finding
        static function plugins_directory() {
            if (WPASDPlugin::determine_ASDEnvironment() == ASDEnvironment::WordPressMU)
                return WP_CONTENT_DIR . '/mu-plugins';
            else
                return WP_CONTENT_DIR . '/plugins';
        }
        
        static function plugins_url() {
           if (WPASDPlugin::determine_ASDEnvironment() == ASDEnvironment::WordPressMU)
               return get_option('siteurl') . '/wp-content/mu-plugins';
           else
               return get_option('siteurl') . '/wp-content/plugins';
        }
        
        static function path_to_plugin_directory() {
            $current_directory = basename(dirname(__FILE__));
            
            return WPASDPlugin::plugins_directory() . "/${current_directory}/..";
        }
        
        static function url_to_plugin_directory() {
           $current_directory = basename(dirname(__FILE__));
           
           return WPASDPlugin::plugins_url() . "/${current_directory}";
        }
        
        static function path_to_plugin($file_path) {
            $file_name = basename($file_path); // /etc/blah/file.txt => file.txt
            
            if (WPASDPlugin::determine_ASDEnvironment() == ASDEnvironment::WordPressMU)
                return WPASDPlugin::plugins_directory() . "/${file_name}";
            else
                return WPASDPlugin::path_to_plugin_directory() . "/${file_name}";
        }
        
        // options
        abstract protected function register_default_options();
        
        // option retrieval
        static function retrieve_options($options_name) {
            if (WPASDPlugin::determine_ASDEnvironment() == ASDEnvironment::WordPressMU || WPASDPlugin::determine_ASDEnvironment() == ASDEnvironment::WordPressMS)
                return get_site_option($options_name);
            else
                return get_option($options_name);
        }
	
	static function uninstall_options($options_name) {
	    unregister_setting("${options_name}_group", $options_name);
	    WPASDPlugin::remove_options($options_name);
	}
        
        static function remove_options($options_name) {
            if (WPASDPlugin::determine_ASDEnvironment() == ASDEnvironment::WordPressMU || WPASDPlugin::determine_ASDEnvironment() == ASDEnvironment::WordPressMS)
                return delete_site_option($options_name);
            else
                return delete_option($options_name);
        }
        
        static function add_options($options_name, $options) {
            if (WPASDPlugin::determine_ASDEnvironment() == ASDEnvironment::WordPressMU || WPASDPlugin::determine_ASDEnvironment() == ASDEnvironment::WordPressMS)
                return add_site_option($options_name, $options);
            else
                return add_option($options_name, $options);
        }
        
        protected function is_multi_blog() {
            return $this->ASDEnvironment != ASDEnvironment::WordPress;
        }
        
        // calls the appropriate 'authority' checking function depending on the ASDEnvironment
        protected function is_authority() {
            if ($this->ASDEnvironment == ASDEnvironment::WordPress)
                return is_admin();
            
            if ($this->ASDEnvironment == ASDEnvironment::WordPressMU)
                return is_site_admin();
            
            if ($this->ASDEnvironment == ASDEnvironment::WordPressMS)
                return is_super_admin();
        }
    }
}

?>